@@ -3,13 +3,13 @@
 
 #include "debuggeritemmanager.h"
 
-#include "debuggeritem.h"
 #include "debuggertr.h"
 
 #include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/icore.h>
 
 #include <projectexplorer/devicesupport/devicemanager.h>
+#include <projectexplorer/kitaspect.h>
 #include <projectexplorer/kitoptionspage.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectexplorericons.h>
@@ -19,13 +19,13 @@
 #include <utils/detailswidget.h>
 #include <utils/environment.h>
 #include <utils/fileutils.h>
+#include <utils/futuresynchronizer.h>
 #include <utils/hostosinfo.h>
 #include <utils/layoutbuilder.h>
 #include <utils/pathchooser.h>
 #include <utils/persistentsettings.h>
-#include <utils/process.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
-#include <utils/treemodel.h>
 #include <utils/winutils.h>
 
 #include <nanotrace/nanotrace.h>
@@ -45,11 +45,49 @@
 #include <QTreeView>
 #include <QWidget>
 
+using namespace Debugger;
 using namespace Debugger::Internal;
 using namespace Core;
 using namespace ProjectExplorer;
 using namespace Utils;
 
+static DebuggerItem makeAutoDetectedDebuggerItem(
+    const FilePath &command,
+    const DebuggerItem::TechnicalData &technicalData,
+    const QString &detectionSource)
+{
+    DebuggerItem item;
+    item.createId();
+    item.setCommand(command);
+    item.setDetectionSource(detectionSource);
+    item.setAutoDetected(true);
+    item.setEngineType(technicalData.engineType);
+    item.setAbis(technicalData.abis);
+    item.setVersion(technicalData.version);
+    const QString name = detectionSource.isEmpty() ? Tr::tr("System %1 at %2")
+                                                   : Tr::tr("Detected %1 at %2");
+    item.setUnexpandedDisplayName(name.arg(item.engineTypeName()).arg(command.toUserOutput()));
+    item.setLastModified(command.lastModified());
+    return item;
+}
+
+static Result<DebuggerItem> makeAutoDetectedDebuggerItem(
+    const FilePath &command, const QString &detectionSource)
+{
+    Result<DebuggerItem::TechnicalData> technicalData
+        = DebuggerItem::TechnicalData::extract(command, {});
+
+    if (!technicalData)
+        return make_unexpected(std::move(technicalData).error());
+
+    return makeAutoDetectedDebuggerItem(command, *technicalData, detectionSource);
+}
+
+static bool doEnableNativeDapDebuggers()
+{
+    return qgetenv("QTC_ENABLE_NATIVE_DAP_DEBUGGERS").toInt() != 0;
+}
+
 namespace Debugger {
 namespace Internal {
 
@@ -101,50 +139,48 @@ private:
 // DebuggerTreeItem
 // --------------------------------------------------------------------------
 
-class DebuggerTreeItem : public TreeItem
+QVariant DebuggerTreeItem::data(int column, int role) const
 {
-public:
-    DebuggerTreeItem(const DebuggerItem &item, bool changed)
-        : m_item(item), m_orig(item), m_added(changed), m_changed(changed)
-    {}
-
-    QVariant data(int column, int role) const override
-    {
-        switch (role) {
-            case Qt::DisplayRole:
-                switch (column) {
-                case 0: return m_item.displayName();
-                case 1: return m_item.command().toUserOutput();
-                case 2: return m_item.engineTypeName();
-                }
-                break;
-
-            case Qt::FontRole: {
-                QFont font;
-                if (m_changed)
-                    font.setBold(true);
-                if (m_removed)
-                    font.setStrikeOut(true);
-                return font;
-            }
-
-            case Qt::DecorationRole:
-                if (column == 0)
-                    return m_item.decoration();
-                break;
-
-            case Qt::ToolTipRole:
-                return m_item.validityMessage();
+    switch (role) {
+    case Qt::DisplayRole:
+        switch (column) {
+        case 0:
+            return m_item.displayName();
+        case 1:
+            return m_item.command().toUserOutput();
+        case 2:
+            return m_item.engineTypeName();
         }
-        return QVariant();
+        break;
+
+    case Qt::FontRole: {
+        QFont font;
+        if (m_changed)
+            font.setBold(true);
+        if (m_removed)
+            font.setStrikeOut(true);
+        return font;
     }
 
-    DebuggerItem m_item; // Displayed, possibly unapplied data.
-    DebuggerItem m_orig; // Stored original data.
-    bool m_added;
-    bool m_changed;
-    bool m_removed = false;
-};
+    case Qt::DecorationRole:
+        if (column == 0)
+            return m_item.decoration();
+        break;
+
+    case Qt::ToolTipRole:
+        return m_item.validityMessage();
+
+    case KitAspect::IdRole:
+        return m_item.id();
+
+    case KitAspect::QualityRole:
+        return int(m_item.problem());
+
+    case KitAspect::IsNoneRole:
+        return !m_item.isValid();
+    }
+    return QVariant();
+}
 
 // --------------------------------------------------------------------------
 // DebuggerItemModel
@@ -378,7 +414,7 @@ DebuggerItemConfigWidget::DebuggerItemConfigWidget()
     // clang-format off
     using namespace Layouting;
     Form {
-        fieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow),
+        fieldGrowthPolicy(int(QFormLayout::AllNonFixedFieldsGrow)),
         Tr::tr("Name:"), m_displayNameLineEdit, br,
         Tr::tr("Path:"), m_binaryChooser, br,
         m_cdbLabel, br,
@@ -480,14 +516,14 @@ void DebuggerItemConfigWidget::binaryPathHasChanged()
     if (!m_generic) {
         m_updateWatcher.cancel();
 
-        DebuggerItem tmp;
         if (m_binaryChooser->filePath().isExecutableFile()) {
-            tmp = item();
-            m_updateWatcher.setFuture(Utils::asyncRun([tmp]() mutable {
+            m_updateWatcher.setFuture(Utils::asyncRun([tmp = item()]() mutable {
                 tmp.reinitializeFromFile();
                 return tmp;
             }));
+            Utils::futureSynchronizer()->addFuture(m_updateWatcher.future());
         } else {
+            const DebuggerItem tmp;
             setAbis(tmp.abiNames());
             m_version->setText(tmp.version());
             m_engineType = tmp.engineType();
@@ -549,9 +585,9 @@ void DebuggerItemModel::autoDetectCdbDebuggers()
     for (const QFileInfo &kitFolderFi : kitFolders) {
         const QString path = kitFolderFi.absoluteFilePath();
         QStringList abis = {"x86", "x64"};
-        if (HostOsInfo::hostArchitecture() == HostOsInfo::HostArchitectureArm64)
+        if (HostOsInfo::hostArchitecture() == Utils::OsArchArm64)
             abis << "arm64";
-        for (const QString &abi: abis) {
+        for (const QString &abi: std::as_const(abis)) {
             const QFileInfo cdbBinary(path + "/Debuggers/" + abi + "/cdb.exe");
             if (cdbBinary.isExecutable())
                 cdbs.append(FilePath::fromString(cdbBinary.absoluteFilePath()));
@@ -606,12 +642,31 @@ static Utils::FilePaths searchGdbPathsFromRegistry()
 }
 
 void DebuggerItemModel::autoDetectGdbOrLldbDebuggers(const FilePaths &searchPaths,
-                                                              const QString &detectionSource,
-                                                              QString *logMessage)
+                                                     const QString &detectionSource,
+                                                     QString *logMessage)
 {
-    const QStringList filters = {"gdb-i686-pc-mingw32", "gdb-i686-pc-mingw32.exe", "gdb",
-                                 "gdb.exe", "lldb", "lldb.exe", "lldb-[1-9]*",
-                                 "arm-none-eabi-gdb-py.exe", "*-*-*-gdb"};
+    QStringList filters
+        = {"gdb-i686-pc-mingw32",
+           "gdb-i686-pc-mingw32.exe",
+           "gdb",
+           "gdb.exe",
+           "lldb",
+           "lldb.exe",
+           "lldb-[1-9]*",
+           "arm-none-eabi-gdb-py.exe",
+           "*-*-*-gdb"};
+
+    if (doEnableNativeDapDebuggers()) {
+        filters.append({
+            "lldb-dap",
+            "lldb-dap.exe",
+            "lldb-dap-*",
+            // LLDB DAP server was named lldb-vscode prior LLVM 18.0.0
+            "lldb-vscode",
+            "lldb-vscode.exe",
+            "lldb-vscode-*",
+        });
+    }
 
     if (searchPaths.isEmpty())
         return;
@@ -620,31 +675,32 @@ void DebuggerItemModel::autoDetectGdbOrLldbDebuggers(const FilePaths &searchPath
 
     if (searchPaths.front().osType() == OsTypeMac) {
         Process proc;
-        proc.setTimeoutS(2);
         proc.setCommand({"xcrun", {"--find", "lldb"}});
-        proc.runBlocking();
-        // FIXME:
+        using namespace std::chrono_literals;
+        proc.runBlocking(2s);
         if (proc.result() == ProcessResult::FinishedWithSuccess) {
-            QString lPath = proc.allOutput().trimmed();
-            if (!lPath.isEmpty()) {
-                const QFileInfo fi(lPath);
-                if (fi.exists() && fi.isExecutable() && !fi.isDir())
-                    suspects.append(FilePath::fromString(fi.absoluteFilePath()));
-            }
+            const FilePath lPath = FilePath::fromUserInput(proc.allOutput().trimmed());
+            if (lPath.isExecutableFile())
+                suspects.append(lPath);
         }
     }
 
     FilePaths paths = searchPaths;
-    if (!searchPaths.front().needsDevice())
+    if (searchPaths.front().isLocal()) {
         paths.append(searchGdbPathsFromRegistry());
 
+        const Result<FilePath> lldb = Core::ICore::lldbExecutable(CLANG_BINDIR);
+        if (lldb)
+            suspects.append(*lldb);
+    }
+
     paths = Utils::filteredUnique(paths);
 
     const auto addSuspect = [&suspects](const FilePath &entry) {
         suspects.append(entry);
         return IterationPolicy::Continue;
     };
-    for (const FilePath &path : paths)
+    for (const FilePath &path : std::as_const(paths))
         path.iterateDirectory(addSuspect, {filters, QDir::Files | QDir::Executable});
 
     QStringList logMessages{Tr::tr("Searching debuggers...")};
@@ -652,24 +708,78 @@ void DebuggerItemModel::autoDetectGdbOrLldbDebuggers(const FilePaths &searchPath
         const auto commandMatches = [command](const DebuggerTreeItem *titem) {
             return titem->m_item.command() == command;
         };
-        if (DebuggerTreeItem *existingItem = findItemAtLevel<2>(commandMatches)) {
-            if (command.lastModified() != existingItem->m_item.lastModified())
-                existingItem->m_item.reinitializeFromFile();
+        if (DebuggerTreeItem *existingTreeItem = findItemAtLevel<2>(commandMatches)) {
+            DebuggerItem &existingItem = existingTreeItem->m_item;
+            if (command.lastModified() != existingItem.lastModified())
+                existingItem.reinitializeFromFile();
+
+            if (doEnableNativeDapDebuggers()) {
+                if (existingItem.engineType() != GdbEngineType)
+                    continue;
+
+                // GDB starting version 14.1.0 supports DAP interface, but unlike LLDB,
+                // it uses the same binary, hence this hack.
+                const QVersionNumber dapSupportMinVersion{14, 1, 0};
+                if (QVersionNumber::fromString(existingItem.version()) < dapSupportMinVersion)
+                    continue;
+                // This is the "update" path: there's already a capable GDB in the settings,
+                // we only need to add a corresponding DAP entry if it's missing.
+                const auto commandMatchesAndIsDap = [command, commandMatches](
+                                                        const DebuggerTreeItem *titem) {
+                    return commandMatches(titem) && titem->m_item.engineType() == GdbDapEngineType;
+                };
+                const DebuggerTreeItem *existingDapTreeItem = findItemAtLevel<2>(
+                    commandMatchesAndIsDap);
+                if (existingDapTreeItem)
+                    continue;
+
+                const DebuggerItem dapItem = makeAutoDetectedDebuggerItem(
+                    command,
+                    {
+                        .engineType = GdbDapEngineType,
+                        .abis = existingItem.abis(),
+                        .version = existingItem.version(),
+                    },
+                    detectionSource);
+                addDebuggerItem(dapItem);
+                logMessages.append(
+                    Tr::tr("Added a surrogate GDB DAP item for existing entry \"%1\".")
+                        .arg(command.toUserOutput()));
+            }
             continue;
         }
-        DebuggerItem item;
-        item.createId();
-        item.setDetectionSource(detectionSource);
-        item.setAutoDetected(true);
-        item.setCommand(command);
-        item.reinitializeFromFile();
-        if (item.engineType() == NoEngineType)
+
+        const Result<DebuggerItem> item
+            = makeAutoDetectedDebuggerItem(command, detectionSource);
+        if (!item) {
+            logMessages.append(item.error());
             continue;
-        //: %1: Debugger engine type (GDB, LLDB, CDB...), %2: Path
-        const QString name = detectionSource.isEmpty() ? Tr::tr("System %1 at %2") : Tr::tr("Detected %1 at %2");
-        item.setUnexpandedDisplayName(name.arg(item.engineTypeName()).arg(command.toUserOutput()));
-        addDebuggerItem(item);
+        }
+
+        addDebuggerItem(*item);
         logMessages.append(Tr::tr("Found: \"%1\"").arg(command.toUserOutput()));
+        if (doEnableNativeDapDebuggers()) {
+            if (item->engineType() != GdbEngineType)
+                continue;
+
+            // GDB starting version 14.1.0 supports DAP interface, but unlike LLDB,
+            // it uses the same binary, hence this hack
+            const QVersionNumber dapSupportMinVersion{14, 1, 0};
+            if (QVersionNumber::fromString(item->version()) < dapSupportMinVersion)
+                continue;
+
+            const DebuggerItem dapItem = makeAutoDetectedDebuggerItem(
+                command,
+                {
+                    .engineType = GdbDapEngineType,
+                    .abis = item->abis(),
+                    .version = item->version(),
+                },
+                detectionSource);
+            addDebuggerItem(dapItem);
+            logMessages.append(
+                Tr::tr("Added a surrogate GDB DAP item for \"%1\".").arg(command.toUserOutput()));
+        }
     }
     if (logMessage)
         *logMessage = logMessages.join('\n');
@@ -701,7 +811,7 @@ void DebuggerItemModel::autoDetectUvscDebuggers()
 
         QString errorMsg;
         const QString uVisionVersion = winGetDLLVersion(
-                    WinDLLFileVersion, uVision.toString(), &errorMsg);
+                    WinDLLFileVersion, uVision.toUrlishString(), &errorMsg);
 
         DebuggerItem item;
         item.createId();
@@ -778,7 +888,7 @@ void DebuggerItemModel::readDebuggers(const FilePath &fileName, bool isSystem)
                     continue;
                 }
                 // FIXME: During startup, devices are not yet available, so we cannot check if the file still exists.
-                if (!item.command().needsDevice() && !item.command().isExecutableFile()) {
+                if (item.command().isLocal() && !item.command().isExecutableFile()) {
                     qWarning() << QString("DebuggerItem \"%1\" (%2) read from \"%3\" dropped since the command is not executable.")
                                   .arg(item.command().toUserOutput(), item.id().toString(), fileName.toUserOutput());
                     continue;
@@ -824,7 +934,7 @@ void DebuggerItemModel::saveDebuggers()
         }
     });
     data.insert(DEBUGGER_COUNT_KEY, count);
-    m_writer.save(data, ICore::dialogParent());
+    m_writer.save(data);
 
     // Do not save default debuggers as they are set by the SDK.
 }
